@@ -1,5 +1,5 @@
 /* Test that openat works.
-   Copyright (C) 2009-2023 Free Software Foundation, Inc.
+   Copyright (C) 2009-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -94,5 +94,5 @@ main ()
     ASSERT (openat (dfd, ".", O_RDONLY) == STDIN_FILENO);
     ASSERT (close (dfd) == 0);
   }
-  return result;
+  return (result ? result : test_exit_status);
 }
