@@ -1,5 +1,5 @@
 /* Public API for GNU gettext PO files - contained in libgettextpo.
-   Copyright (C) 2003-2008, 2010, 2012-2016, 2019-2023 Free Software Foundation, Inc.
+   Copyright (C) 2003-2008, 2010, 2012-2016, 2019-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software: you can redistribute it and/or modify
@@ -28,7 +28,7 @@ extern "C" {
 /* =========================== Meta Information ============================ */
 
 /* Version number: (major<<16) + (minor<<8) + subminor */
-#define LIBGETTEXTPO_VERSION 0x001602
+#define LIBGETTEXTPO_VERSION 0x001700
 extern int libgettextpo_version;
 
 /* ================================= Types ================================= */
@@ -46,7 +46,7 @@ typedef struct po_message *po_message_t;
 /* A po_filepos_t represents a string's position within a source file.  */
 typedef struct po_filepos *po_filepos_t;
 
-/* A po_error_handler handles error situations.  */
+/* A po_error_handler handles error situations.  No longer used.  */
 struct po_error_handler
 {
   /* Signal an error.  The error message is built from FORMAT and the following
@@ -298,8 +298,11 @@ extern void po_message_set_fuzzy (po_message_t message, int fuzzy);
    type (e.g. "c-format").  */
 extern int po_message_is_format (po_message_t message, const char *format_type);
 
-/* Change the format string mark for a given type of a message.  */
-extern void po_message_set_format (po_message_t message, const char *format_type, /*bool*/int value);
+/* Change the format string mark for a given type of a message.
+   Pass value = 1 to assert the format string mark (e.g. "c-format"),
+   value = 0 to assert the opposite (leading to e.g. "no-c-format"),
+   or value = -1 to remove the format string mark and its opposite.  */
+extern void po_message_set_format (po_message_t message, const char *format_type, int value);
 
 /* If a numeric range of a message is set, return true and store the minimum
    and maximum value in *MINP and *MAXP.  */
