@@ -1,6 +1,5 @@
 /* Creates an English translation catalog.
-   Copyright (C) 2001-2007, 2009-2010, 2012, 2014, 2016, 2018-2022 Free Software
-   Foundation, Inc.
+   Copyright (C) 2001-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -29,25 +28,29 @@
 
 #include <textstyle.h>
 
+#include <error.h>
 #include "noreturn.h"
 #include "closeout.h"
 #include "dir-list.h"
-#include "error.h"
 #include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
 #include "basename-lgpl.h"
 #include "message.h"
-#include "read-catalog.h"
+#include "read-catalog-file.h"
 #include "read-po.h"
 #include "read-properties.h"
 #include "read-stringtable.h"
 #include "msgl-english.h"
+#include "msgl-ascii.h"
 #include "msgl-header.h"
+#include "c-strstr.h"
+#include "xalloc.h"
 #include "write-catalog.h"
 #include "write-po.h"
 #include "write-properties.h"
 #include "write-stringtable.h"
+#include "xerror-handler.h"
 #include "propername.h"
 #include "gettext.h"
 
@@ -88,6 +91,7 @@ static const struct option long_options[] =
 
 /* Forward declaration of local functions.  */
 _GL_NORETURN_FUNC static void usage (int status);
+static msgdomain_list_ty *fill_header (msgdomain_list_ty *mdlp);
 
 
 int
@@ -108,6 +112,7 @@ main (int argc, char **argv)
   /* Set program name for messages.  */
   set_program_name (argv[0]);
   error_print_progname = maybe_print_progname;
+  gram_max_allowed_errors = 20;
 
   /* Set locale via LC_ALL.  */
   setlocale (LC_ALL, "");
@@ -241,7 +246,7 @@ License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "2001-2022", "https://gnu.org/licenses/gpl.html");
+              "2001-2024", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -270,6 +275,10 @@ There is NO WARRANTY, to the extent permitted by law.\n\
   /* Read input file.  */
   result = read_catalog_file (argv[optind], input_syntax);
 
+  if (!output_syntax->requires_utf8)
+    /* Fill the header entry.  */
+    result = fill_header (result);
+
   /* Add English translations.  */
   result = msgdomain_list_english (result);
 
@@ -284,7 +293,8 @@ There is NO WARRANTY, to the extent permitted by law.\n\
     msgdomain_list_set_header_field (result, "Language:", catalogname);
 
   /* Write the merged message list out.  */
-  msgdomain_list_print (result, output_file, output_syntax, force_po, false);
+  msgdomain_list_print (result, output_file, output_syntax,
+                        textmode_xerror_handler, force_po, false);
 
   exit (EXIT_SUCCESS);
 }
@@ -397,3 +407,63 @@ or by email to <%s>.\n"),
 
   exit (status);
 }
+
+/* Fill the templates in the most essential fields of the header entry,
+   namely to force a charset name.  */
+static msgdomain_list_ty *
+fill_header (msgdomain_list_ty *mdlp)
+{
+  size_t k, j;
+
+  if (mdlp->encoding == NULL
+      && is_ascii_msgdomain_list (mdlp))
+    mdlp->encoding = "ASCII";
+
+  if (mdlp->encoding != NULL)
+    for (k = 0; k < mdlp->nitems; k++)
+      {
+        message_list_ty *mlp = mdlp->item[k]->messages;
+
+        if (mlp->nitems > 0)
+          {
+            message_ty *header_mp = NULL;
+
+            /* Search the header entry.  */
+            for (j = 0; j < mlp->nitems; j++)
+              if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
+                {
+                  header_mp = mlp->item[j];
+                  break;
+                }
+
+            /* If it wasn't found, provide one.  */
+            if (header_mp == NULL)
+              {
+                static lex_pos_ty pos = { __FILE__, __LINE__ };
+                const char *msgstr =
+                  "Content-Type: text/plain; charset=CHARSET\n"
+                  "Content-Transfer-Encoding: 8bit\n";
+
+                header_mp =
+                  message_alloc (NULL, "", NULL, msgstr, strlen (msgstr), &pos);
+                message_list_prepend (mlp, header_mp);
+              }
+
+            /* Fill in the charset name.  */
+            {
+              const char *header = header_mp->msgstr;
+              const char *charsetstr = c_strstr (header, "charset=");
+              if (charsetstr != NULL)
+                {
+                  charsetstr += strlen ("charset=");
+                  header_set_charset (header_mp, charsetstr, mdlp->encoding);
+                }
+            }
+
+            /* Finally remove the fuzzy attribute.  */
+            header_mp->is_fuzzy = false;
+          }
+      }
+
+  return mdlp;
+}
