@@ -1,5 +1,5 @@
 /* Initializes a new PO file.
-   Copyright (C) 2001-2020 Free Software Foundation, Inc.
+   Copyright (C) 2001-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 #endif
 #include <alloca.h>
 
+#include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -31,39 +32,27 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h>
 #include <sys/types.h>
 
 #if HAVE_PWD_H
 # include <pwd.h>
 #endif
 
-#include <unistd.h>
-
-#if HAVE_DIRENT_H
-# include <dirent.h>
-#endif
-
-#if HAVE_DIRENT_H
-# define HAVE_DIR 1
-#else
-# define HAVE_DIR 0
-#endif
-
 #include <textstyle.h>
 
-/* Get BINDIR.  */
-#include "configmake.h"
+#include <error.h>
 
+#include "noreturn.h"
 #include "closeout.h"
-#include "error.h"
 #include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
-#include "basename.h"
+#include "basename-lgpl.h"
 #include "c-strstr.h"
 #include "c-strcase.h"
 #include "message.h"
-#include "read-catalog.h"
+#include "read-catalog-file.h"
 #include "read-po.h"
 #include "read-properties.h"
 #include "read-stringtable.h"
@@ -71,6 +60,8 @@
 #include "write-po.h"
 #include "write-properties.h"
 #include "write-stringtable.h"
+#include "msgl-charset.h"
+#include "xerror-handler.h"
 #include "po-charset.h"
 #include "localcharset.h"
 #include "localename.h"
@@ -136,11 +127,7 @@ static const struct option long_options[] =
 };
 
 /* Forward declaration of local functions.  */
-static void usage (int status)
-#if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
-     __attribute__ ((noreturn))
-#endif
-;
+_GL_NORETURN_FUNC static void usage (int status);
 static const char *find_pot (void);
 static const char *catalogname_for_locale (const char *locale);
 static const char *language_of_locale (const char *locale);
@@ -164,6 +151,7 @@ main (int argc, char **argv)
   /* Set program name for messages.  */
   set_program_name (argv[0]);
   error_print_progname = maybe_print_progname;
+  gram_max_allowed_errors = 20;
 
   /* Set locale via LC_ALL.  */
   setlocale (LC_ALL, "");
@@ -266,14 +254,15 @@ main (int argc, char **argv)
   /* Version information is requested.  */
   if (do_version)
     {
-      printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
+      printf ("%s (GNU %s) %s\n", last_component (program_name),
+              PACKAGE, VERSION);
       /* xgettext: no-wrap */
       printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
 License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "2001-2020", "https://gnu.org/licenses/gpl.html");
+              "2001-2024", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -336,48 +325,7 @@ the output .po file through the --output-file option.\n"),
 
   /* Read input file.  */
   result = read_catalog_file (input_file, input_syntax);
-
-#if defined _WIN32 || defined __CYGWIN__
-  /* The function fill_header invokes, directly or indirectly, some programs
-     that are installed in ${libdir}/gettext:
-       - hostname, invoked indirectly through 'user-email'.
-       - urlget, invoked indirectly through 'team-address'.
-       - cldr-plurals, invoked directly.
-     These programs depend on libintl.  In installations with shared libraries,
-     we need to guarantee that the programs find the DLL, which is installed
-     in ${bindir}, not in ${libdir}/gettext.  The preferred way to do so is to
-     extend $PATH, so that it contains ${bindir}.  */
-  {
-    const char *orig_path;
-    size_t orig_path_len;
-    char separator;
-    const char *bindir;
-    size_t bindir_len;
-    char *augmented_path;
-
-    orig_path = getenv ("PATH");
-    if (orig_path == NULL)
-      orig_path = "";
-    orig_path_len = strlen (orig_path);
-
-    #if defined __CYGWIN__
-    separator = ':';
-    #else /* native Windows */
-    separator = ';';
-    #endif
-
-    bindir = BINDIR;
-    bindir_len = strlen (bindir);
-
-    /* Concatenate bindir, separator, orig_path.  */
-    augmented_path = XNMALLOC (bindir_len + 1 + orig_path_len + 1, char);
-    memcpy (augmented_path, bindir, bindir_len);
-    augmented_path[bindir_len] = separator;
-    memcpy (augmented_path + bindir_len + 1, orig_path, orig_path_len + 1);
-
-    xsetenv ("PATH", augmented_path, 1);
-  }
-#endif
+  check_pot_charset (result, input_file);
 
   /* Fill the header entry.  */
   result = fill_header (result);
@@ -389,7 +337,8 @@ the output .po file through the --output-file option.\n"),
     result = update_msgstr_plurals (result);
 
   /* Write the modified message list out.  */
-  msgdomain_list_print (result, output_file, output_syntax, true, false);
+  msgdomain_list_print (result, output_file, output_syntax,
+                        textmode_xerror_handler, true, false);
 
   if (!no_translator)
     fprintf (stderr, "\n");
@@ -493,7 +442,6 @@ or by email to <%s>.\n"),
 static const char *
 find_pot ()
 {
-#if HAVE_DIR
   DIR *dirp;
   char *found = NULL;
 
@@ -536,7 +484,6 @@ Please specify the input .pot file through the --input option.\n")));
       if (found != NULL)
         return found;
     }
-#endif
 
   multiline_error (xstrdup (""),
                    xstrdup (_("\
@@ -577,6 +524,7 @@ catalogname_for_locale (const char *locale)
     "bem_ZM",   /* Bemba        Zambia */
     "bg_BG",    /* Bulgarian    Bulgaria */
     "bho_IN",   /* Bhojpuri     India */
+    "bi_VU",    /* Bislama      Vanuatu */
     "bik_PH",   /* Bikol        Philippines */
     "bin_NG",   /* Bini         Nigeria */
     "bm_ML",    /* Bambara      Mali */
@@ -655,6 +603,7 @@ catalogname_for_locale (const char *locale)
     "kok_IN",   /* Konkani      India */
     "kr_NG",    /* Kanuri       Nigeria */
     "kru_IN",   /* Kurukh       India */
+    "ky_KG",    /* Kyrgyz       Kyrgyzstan */
     "lg_UG",    /* Ganda        Uganda */
     "li_BE",    /* Limburgish   Belgium */
     "lo_LA",    /* Laotian      Laos */
@@ -669,10 +618,11 @@ catalogname_for_locale (const char *locale)
     "mak_ID",   /* Makasar      Indonesia */
     "man_ML",   /* Mandingo     Mali */
     "men_SL",   /* Mende        Sierra Leone */
+    "mfe_MU",   /* Mauritian Creole     Mauritius */
     "mg_MG",    /* Malagasy     Madagascar */
     "mi_NZ",    /* Maori        New Zealand */
     "min_ID",   /* Minangkabau  Indonesia */
-    "mk_MK",    /* Macedonian   Macedonia */
+    "mk_MK",    /* Macedonian   North Macedonia */
     "ml_IN",    /* Malayalam    India */
     "mn_MN",    /* Mongolian    Mongolia */
     "mni_IN",   /* Manipuri     India */
@@ -693,6 +643,7 @@ catalogname_for_locale (const char *locale)
     "no_NO",    /* Norwegian    Norway */
     "nr_ZA",    /* South Ndebele        South Africa */
     "nso_ZA",   /* Northern Sotho       South Africa */
+    "ny_MW",    /* Chichewa     Malawi */
     "nym_TZ",   /* Nyamwezi     Tanzania */
     "nyn_UG",   /* Nyankole     Uganda */
     "oc_FR",    /* Occitan      France */
@@ -711,6 +662,7 @@ catalogname_for_locale (const char *locale)
     "rn_BI",    /* Kirundi      Burundi */
     "ro_RO",    /* Romanian     Romania */
     "ru_RU",    /* Russian      Russia */
+    "rw_RW",    /* Kinyarwanda  Rwanda */
     "sa_IN",    /* Sanskrit     India */
     "sah_RU",   /* Yakut        Russia */
     "sas_ID",   /* Sasak        Indonesia */
@@ -741,7 +693,8 @@ catalogname_for_locale (const char *locale)
     "tk_TM",    /* Turkmen      Turkmenistan */
     "tl_PH",    /* Tagalog      Philippines */
     "to_TO",    /* Tonga        Tonga */
-    "tr_TR",    /* Turkish      Turkey */
+    "tpi_PG",   /* Tok Pisin    Papua New Guinea */
+    "tr_TR",    /* Turkish      Türkiye */
     "tum_MW",   /* Tumbuka      Malawi */
     "ug_CN",    /* Uighur       China */
     "uk_UA",    /* Ukrainian    Ukraine */
@@ -920,7 +873,7 @@ project_id (const char *header)
   {
     const char *gettextlibdir;
     char *prog;
-    char *argv[3];
+    const char *argv[3];
     pid_t child;
     int fd[1];
     FILE *fp;
@@ -929,9 +882,9 @@ project_id (const char *header)
     size_t linelen;
     int exitstatus;
 
-    gettextlibdir = getenv ("GETTEXTLIBDIR_SRCDIR");
+    gettextlibdir = getenv ("GETTEXTLIBEXECDIR_SRCDIR");
     if (gettextlibdir == NULL || gettextlibdir[0] == '\0')
-      gettextlibdir = relocate (LIBDIR "/gettext");
+      gettextlibdir = relocate (LIBEXECDIR "/gettext");
 
     prog = xconcatenated_filename (gettextlibdir, "project-id", NULL);
 
@@ -939,8 +892,8 @@ project_id (const char *header)
     argv[0] = BOURNE_SHELL;
     argv[1] = prog;
     argv[2] = NULL;
-    child = create_pipe_in (prog, BOURNE_SHELL, argv, DEV_NULL, false, true,
-                            false, fd);
+    child = create_pipe_in (prog, BOURNE_SHELL, argv, NULL, NULL,
+                            DEV_NULL, false, true, false, fd);
     if (child == -1)
       goto failed;
 
@@ -1002,7 +955,7 @@ project_id_version (const char *header)
   {
     const char *gettextlibdir;
     char *prog;
-    char *argv[4];
+    const char *argv[4];
     pid_t child;
     int fd[1];
     FILE *fp;
@@ -1011,9 +964,9 @@ project_id_version (const char *header)
     size_t linelen;
     int exitstatus;
 
-    gettextlibdir = getenv ("GETTEXTLIBDIR_SRCDIR");
+    gettextlibdir = getenv ("GETTEXTLIBEXECDIR_SRCDIR");
     if (gettextlibdir == NULL || gettextlibdir[0] == '\0')
-      gettextlibdir = relocate (LIBDIR "/gettext");
+      gettextlibdir = relocate (LIBEXECDIR "/gettext");
 
     prog = xconcatenated_filename (gettextlibdir, "project-id", NULL);
 
@@ -1022,8 +975,8 @@ project_id_version (const char *header)
     argv[1] = prog;
     argv[2] = "yes";
     argv[3] = NULL;
-    child = create_pipe_in (prog, BOURNE_SHELL, argv, DEV_NULL, false, true,
-                            false, fd);
+    child = create_pipe_in (prog, BOURNE_SHELL, argv, NULL, NULL,
+                            DEV_NULL, false, true, false, fd);
     if (child == -1)
       goto failed;
 
@@ -1173,8 +1126,9 @@ get_user_email ()
      "msginit.exe: subprocess ... failed: No such file or directory"  */
 #if !(defined _WIN32 && ! defined __CYGWIN__)
   {
-    const char *prog = relocate (LIBDIR "/gettext/user-email");
-    char *argv[4];
+    const char *prog = relocate (LIBEXECDIR "/gettext/user-email");
+    const char *dll_dirs[2];
+    const char *argv[4];
     pid_t child;
     int fd[1];
     FILE *fp;
@@ -1183,16 +1137,24 @@ get_user_email ()
     size_t linelen;
     int exitstatus;
 
+    /* The program 'hostname', that 'user-email' may invoke, is installed in
+       gettextlibdir and depends on libintl and libgettextlib.  On Windows,
+       in installations with shared libraries, these DLLs are installed in
+       ${bindir}.  Make sure that the program can find them, even if
+       ${bindir} is not in $PATH.  */
+    dll_dirs[0] = relocate (BINDIR);
+    dll_dirs[1] = NULL;
+
     /* Ask the user for his email address.  */
     argv[0] = BOURNE_SHELL;
-    argv[1] = (char *) prog;
-    argv[2] = (char *) _("\
+    argv[1] = prog;
+    argv[2] = _("\
 The new message catalog should contain your email address, so that users can\n\
 give you feedback about the translations, and so that maintainers can contact\n\
 you in case of unexpected technical problems.\n");
     argv[3] = NULL;
-    child = create_pipe_in (prog, BOURNE_SHELL, argv, DEV_NULL, false, true,
-                            false, fd);
+    child = create_pipe_in (prog, BOURNE_SHELL, argv, dll_dirs, NULL,
+                            DEV_NULL, false, true, false, fd);
     if (child == -1)
       goto failed;
 
@@ -1280,25 +1242,35 @@ language_team_address ()
 #if !(defined _WIN32 && ! defined __CYGWIN__)
   {
     const char *prog = relocate (PROJECTSDIR "/team-address");
-    char *argv[7];
+    const char *dll_dirs[2];
+    const char *argv[7];
     pid_t child;
     int fd[1];
     FILE *fp;
     char *line;
     size_t linesize;
     size_t linelen;
+    const char *result;
     int exitstatus;
 
+    /* The program 'urlget', that 'team-address' may invoke, is installed in
+       gettextlibdir and depends on libintl and libgettextlib.  On Windows,
+       in installations with shared libraries, these DLLs are installed in
+       ${bindir}.  Make sure that the program can find them, even if
+       ${bindir} is not in $PATH.  */
+    dll_dirs[0] = relocate (BINDIR);
+    dll_dirs[1] = NULL;
+
     /* Call the team-address shell script.  */
     argv[0] = BOURNE_SHELL;
-    argv[1] = (char *) prog;
-    argv[2] = (char *) relocate (PROJECTSDIR);
-    argv[3] = (char *) relocate (LIBDIR "/gettext");
-    argv[4] = (char *) catalogname;
-    argv[5] = (char *) language;
+    argv[1] = prog;
+    argv[2] = relocate (PROJECTSDIR);
+    argv[3] = relocate (LIBEXECDIR "/gettext");
+    argv[4] = catalogname;
+    argv[5] = language;
     argv[6] = NULL;
-    child = create_pipe_in (prog, BOURNE_SHELL, argv, DEV_NULL, false, true,
-                            false, fd);
+    child = create_pipe_in (prog, BOURNE_SHELL, argv, dll_dirs, NULL,
+                            DEV_NULL, false, true, false, fd);
     if (child == -1)
       goto failed;
 
@@ -1313,9 +1285,13 @@ language_team_address ()
     line = NULL; linesize = 0;
     linelen = getline (&line, &linesize, fp);
     if (linelen == (size_t)(-1))
-      line = "";
-    else if (linelen > 0 && line[linelen - 1] == '\n')
-      line[linelen - 1] = '\0';
+      result = "";
+    else
+      {
+        if (linelen > 0 && line[linelen - 1] == '\n')
+          line[linelen - 1] = '\0';
+        result = line;
+      }
 
     fclose (fp);
 
@@ -1328,7 +1304,7 @@ language_team_address ()
         goto failed;
       }
 
-    return line;
+    return result;
   }
 
 failed:
@@ -1430,8 +1406,10 @@ plural_forms ()
   if (gettextcldrdir != NULL && gettextcldrdir[0] != '\0')
     {
       const char *gettextlibdir;
-      char *dirs[3], *last_dir;
-      char *argv[4];
+      const char *dirs[3];
+      char *last_dir;
+      const char *dll_dirs[2];
+      const char *argv[4];
       pid_t child;
       int fd[1];
       FILE *fp;
@@ -1440,9 +1418,9 @@ plural_forms ()
       size_t linelen;
       int exitstatus;
 
-      gettextlibdir = getenv ("GETTEXTLIBDIR_BUILDDIR");
+      gettextlibdir = getenv ("GETTEXTLIBEXECDIR_BUILDDIR");
       if (gettextlibdir == NULL || gettextlibdir[0] == '\0')
-        gettextlibdir = relocate (LIBDIR "/gettext");
+        gettextlibdir = relocate (LIBEXECDIR "/gettext");
 
       prog = xconcatenated_filename (gettextlibdir, "cldr-plurals", EXEEXT);
 
@@ -1457,17 +1435,24 @@ plural_forms ()
           last_dir = dir;
         }
 
+      /* The program 'cldr-plurals', that we invoke here, is installed in
+         gettextlibdir and depends on libintl and libgettextlib.  On Windows,
+         in installations with shared libraries, these DLLs are installed in
+         ${bindir}.  Make sure that the program can find them, even if
+         ${bindir} is not in $PATH.  */
+      dll_dirs[0] = relocate (BINDIR);
+      dll_dirs[1] = NULL;
+
       /* Call the cldr-plurals command.
          argv[0] must be prog, not just the base name "cldr-plurals",
          because on Cygwin in a build with --enable-shared, the libtool
          wrapper of cldr-plurals.exe apparently needs this.  */
       argv[0] = prog;
-      argv[1] = (char *) language;
+      argv[1] = language;
       argv[2] = last_dir;
       argv[3] = NULL;
-      child = create_pipe_in (prog, prog, argv, DEV_NULL,
-                              false, true, false,
-                              fd);
+      child = create_pipe_in (prog, prog, argv, dll_dirs, NULL,
+                              DEV_NULL, false, true, false, fd);
       free (last_dir);
       if (child == -1)
         goto failed;
