@@ -1,5 +1,5 @@
 /* Message list charset and locale charset handling.
-   Copyright (C) 2001-2003, 2005-2007, 2009, 2019 Free Software Foundation, Inc.
+   Copyright (C) 2001-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -28,11 +28,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <error.h>
+#include "msgl-ascii.h"
 #include "po-charset.h"
 #include "localcharset.h"
-#include "error.h"
 #include "progname.h"
-#include "basename.h"
+#include "basename-lgpl.h"
 #include "xmalloca.h"
 #include "xerror.h"
 #include "xvasprintf.h"
@@ -42,6 +43,65 @@
 
 #define _(str) gettext (str)
 
+/* Check whether the POT file's encoding is ASCII or UTF-8.  Otherwise
+   emit a warning.
+   Rationale: A POT file is routinely copied by a translator to a PO file.
+   If a POT file contains non-ASCII messages (or comments) in an encoding
+   other than UTF-8, the translator will most likely encounter trouble adding
+   her own translations in the same encoding.  A translator should not have
+   to convert the POT file to UTF-8 first; instead, the POT file should
+   already be prepeared ready-to-use.  */
+void
+check_pot_charset (const msgdomain_list_ty *mdlp, const char *filename)
+{
+  size_t j, k;
+
+  for (k = 0; k < mdlp->nitems; k++)
+    {
+      const message_list_ty *mlp = mdlp->item[k]->messages;
+
+      for (j = 0; j < mlp->nitems; j++)
+        if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
+          {
+            const char *header = mlp->item[j]->msgstr;
+
+            if (header != NULL)
+              {
+                const char *charsetstr = c_strstr (header, "charset=");
+
+                if (charsetstr != NULL)
+                  {
+                    size_t len;
+                    char *charset;
+
+                    charsetstr += strlen ("charset=");
+                    len = strcspn (charsetstr, " \t\n");
+                    charset = (char *) xmalloca (len + 1);
+                    memcpy (charset, charsetstr, len);
+                    charset[len] = '\0';
+
+                    const char *canon_charset = po_charset_canonicalize (charset);
+
+                    /* "CHARSET" is often used as a placeholder, equivalent
+                       to "any" or "ASCII".  */
+                    if (!(strcmp (charset, "CHARSET") == 0)
+                        && canon_charset == NULL)
+                      error (EXIT_FAILURE, 0,
+                             _("%s: The present charset \"%s\" is not a portable encoding name."),
+                             filename, charset);
+                    if (!is_ascii_message_list (mlp)
+                        && canon_charset != po_charset_utf8)
+                      error (EXIT_FAILURE, 0,
+                             _("%s: The file contains non-ASCII characters but the present charset \"%s\" is not %s."),
+                             filename, charset, "UTF-8");
+
+                    freea (charset);
+                  }
+              }
+          }
+    }
+}
+
 void
 compare_po_locale_charsets (const msgdomain_list_ty *mdlp)
 {
@@ -88,34 +148,35 @@ compare_po_locale_charsets (const msgdomain_list_ty *mdlp)
                     freea (charset);
                     if (canon_locale_code != canon_charset)
                       {
-                        multiline_warning (xasprintf (_("warning: ")),
-                                           xasprintf (_("\
+                        size_t prefix_width =
+                          multiline_warning (xasprintf (_("warning: ")),
+                                             xasprintf (_("\
 Locale charset \"%s\" is different from\n\
 input file charset \"%s\".\n\
 Output of '%s' might be incorrect.\n\
 Possible workarounds are:\n\
-"), locale_code, canon_charset, basename (program_name)));
-                        multiline_warning (NULL,
-                                           xasprintf (_("\
+"), locale_code, canon_charset, last_component (program_name)));
+                        multiline_append (prefix_width,
+                                          xasprintf (_("\
 - Set LC_ALL to a locale with encoding %s.\n\
 "), canon_charset));
                         if (canon_locale_code != NULL)
-                          multiline_warning (NULL,
-                                             xasprintf (_("\
+                          multiline_append (prefix_width,
+                                            xasprintf (_("\
 - Convert the translation catalog to %s using 'msgconv',\n\
   then apply '%s',\n\
   then convert back to %s using 'msgconv'.\n\
-"), canon_locale_code, basename (program_name), canon_charset));
+"), canon_locale_code, last_component (program_name), canon_charset));
                         if (strcmp (canon_charset, "UTF-8") != 0
                             && (canon_locale_code == NULL
                                 || strcmp (canon_locale_code, "UTF-8") != 0))
-                          multiline_warning (NULL,
-                                             xasprintf (_("\
+                          multiline_append (prefix_width,
+                                            xasprintf (_("\
 - Set LC_ALL to a locale with encoding %s,\n\
   convert the translation catalog to %s using 'msgconv',\n\
   then apply '%s',\n\
   then convert back to %s using 'msgconv'.\n\
-"), "UTF-8", "UTF-8", basename (program_name), canon_charset));
+"), "UTF-8", "UTF-8", last_component (program_name), canon_charset));
                         warned = true;
                       }
                   }
@@ -128,5 +189,5 @@ Possible workarounds are:\n\
 Locale charset \"%s\" is not a portable encoding name.\n\
 Output of '%s' might be incorrect.\n\
 A possible workaround is to set LC_ALL=C.\n\
-"), locale_code, basename (program_name)));
+"), locale_code, last_component (program_name)));
 }
